import matplotlib.pyplot as plt
import tensorflow as tf
import graph_builder as bd

# Parameters
# Input parameters
IMG_SIZE = 28
N_PIXELS= IMG_SIZE**2
IMG_SHAPE = (IMG_SIZE, IMG_SIZE)
N_CHANNELS = 1
N_CLASSES = 10

# Network parameters
# Convolutional Layer 1.
FILTER_SIZE1 = 5          # Convolution filters are 5 x 5 pixels.
N_FILTERS1 = 16         # There are 16 of these filters.

# Convolutional Layer 2.
FILTER_SIZE2 = 5          # Convolution filters are 5 x 5 pixels.
N_FILTERS2 = 36         # There are 36 of these filters.

# Fully-connected layer.
C_SIZE = 128             # Number of neurons in fully-connected layer.

def placeholder_inputs(batch_size):
    img_placeholder_flat = tf.placeholder(tf.float32, shape = [batch_size,
                                                          N_PIXELS])
    img_placeholder = tf.reshape(img_placeholder_flat, [-1, IMG_SIZE, IMG_SIZE, N_CHANNELS])
    lab_placeholder = tf.placeholder(tf.int32, shape = [batch_size])
    prob_placeholder = tf.placeholder(tf.float32)
    return img_placeholder, img_placeholder_flat, lab_placeholder, prob_placeholder

def plot_images(images, lab_true, lab_pred = None):
    assert len(images) == len(lab_true) == 9
    fig, axes = plt.subplots(3, 3)

    for i, ax in enumerate(axes.flat):
        ax.imshow(images[i].reshape(IMG_SHAPE), cmap = 'binary')

        if lab_pred is None:
            xlabel = "True: {0}".format(lab_true[i])
        else:
            xlabel = "True: {0}, Pred: {1}".format(lab_true[i], lab_pred[i])

        # Show the classes as the label on the x-axis.
        ax.set_xlabel(xlabel)

        # Remove ticks from the plot.
        ax.set_xticks([])
        ax.set_yticks([])


    plt.show()

def inference(img_placeholder, prob_placeholder):

    # First convolutional layer
    conv1 = bd.new_conv_layer(img_placeholder,
                                 num_input_channels = N_CHANNELS,
                                 filter_size = FILTER_SIZE1,
                                 num_filters = N_FILTERS1,
                                 layer_name = 'Conv_1')

    # Second convolutional layer
    conv2 = bd.new_conv_layer(input = conv1,
                                 num_input_channels= N_FILTERS1,
                                 filter_size = FILTER_SIZE2,
                                 num_filters = N_FILTERS2,
                                 layer_name = 'Conv_2')
    # Flatten layer
    layer_flat, num_features = bd.flatten_layer(conv2)

    # First full-connected layer
    layer_fc1 = bd.new_fc_layer(input = layer_flat,
                                num_inputs = num_features,
                                num_outputs = C_SIZE,
                                layer_name = 'FC_1')

    # Dropout layer
    layer_drop = tf.nn.dropout(layer_fc1, prob_placeholder)

    logits = bd.new_fc_layer(input = layer_drop,
                             num_inputs = C_SIZE,
                             num_outputs = N_CLASSES,
                             layer_name = "FC_2")

    return logits

def loss(logits, labels):
    cross_entropy = tf.nn.sparse_softmax_cross_entropy_with_logits(logits,
                                                                   labels,
                                                                   name = 'xentropy_mean')
    loss = tf.reduce_mean(cross_entropy, name = 'xentropy_mean')
    return(loss)

def training(loss, learning_rate):
    optimizer = tf.train.AdamOptimizer(learning_rate = learning_rate)
    global_step = tf.Variable(0, name = 'global_step', trainable = False)
    train = optimizer.minimize(loss, global_step = global_step)
    return train

def evaluation(logits, labels):
    correct = tf.nn.in_top_k(logits, labels, 1)
    return tf.reduce_sum((tf.cast(correct, tf.int32)))